@@ -128,11 +128,21 @@ calculate_multi_roc <- function(data, M_string, D_string){
 
 #' Check that D is suitable for using as binary disease status
 #' 
-#' Checks for two classes and gives a warning message indicating which level is assumed to be 0/1
+#' Checks for two classes and gives a warning message indicating which level is assumed to be 0/1. Throws an error if more than two levels appear in D.
 #' 
 #' @param D Vector that will be checked for 2-class labels
 #' 
-#' @keywords Internal
+#' @return A vector the same length as D that takes values 0, indicating no disease or 1 indicating disease. 
+#' 
+#' @export
+#' @examples 
+#' 
+#' verify_d(c(1, 0, 1))
+#' \dontrun{
+#' verify_d(c(TRUE, FALSE, TRUE)) #warning
+#' verify_d(c("Dead", "Alive", "Dead")) #warning
+#' verify_d(c("Disease", "Healthy", "Missing")) #error
+#' }
 #' 
 verify_d <- function(D){
 
@@ -146,13 +156,13 @@ verify_d <- function(D){
   zero1 <- c(0, 1)
   names(zero1) <- slev
   
-  zero1[D]
+  zero1[as.character(D)]
   
 }
 
 #' Transform biomarkers stored as wide to long
 #' 
-#' Multple biomarkers measured on the same subjects are often stored as multiple columns in a data frame. This is a convenience function that transforms the data into long format, suitable for use with ggplot and \link{geom_roc}
+#' Multiple biomarkers measured on the same subjects are often stored as multiple columns in a data frame. This is a convenience function that transforms the data into long format, suitable for use with ggplot and \link{geom_roc}
 #' 
 #' @param data Data frame containing disease status and biomarkers stored in columns
 #' @param d Column containing binary disease status. Can be a column name or index
