@@ -1,3 +1,32 @@
+
+#' Key for ROC geom
+#' 
+#' @keywords Internal
+#' @param data Data created by stat
+#' @param params parameters
+#' @param size Size
+#' 
+roc_key <- function(data, params, size) {
+  
+  grobTree(
+      draw_key_path(data, params, size), 
+      pointsGrob(0.5, 0.5,
+                 pch = data$shape,
+                 size = unit(data$size * .pt * 2, "pt"),
+                 gp = gpar(
+                   col = alpha(data$colour, data$alpha),
+                   fontsize = data$size * .pt * 4,
+                   lwd = data$stroke 
+                 )
+      )
+    )
+  
+  
+}
+
+
+
+
 #' @export
 #' @rdname stat_roc
 
@@ -16,9 +45,9 @@ StatRoc <- ggproto("StatRoc", Stat,
                      } else {
                        data$group <- -1L
                      } 
-                   data
+                     data
                    },
-                   compute_group = function(data, scales, na.rm = TRUE){
+                   compute_group = function(data, scales, na.rm = TRUE, max.num.points = 1e3, increasing = TRUE){
                      
                      if(na.rm){
                        data <- subset(data, !is.na(d) & !is.na(m))
@@ -26,7 +55,7 @@ StatRoc <- ggproto("StatRoc", Stat,
                      
                      D <- data$d
                      
-                     T.order <- order(data$m, decreasing=TRUE)
+                     T.order <- order(data$m, decreasing=increasing) ## this is confusing but think about it for a sec
                      TTT <- data$m[T.order]
                      TPF <- cumsum(D[T.order] == 1)
                      FPF <- cumsum(D[T.order] == 0)
@@ -35,23 +64,36 @@ StatRoc <- ggproto("StatRoc", Stat,
                      ## Highest cutoff (Infinity) corresponds to tp=0, fp=0
                      
                      dups <- rev(duplicated(rev(TTT)))
-                     tp <- c(0, TPF[!dups])/sum(D == 1)
-                     fp <- c(0, FPF[!dups])/sum(D == 0)
-                
-                     cutoffs <- c(Inf, TTT[!dups])
+                     TPF <- TPF[!dups]
+                     FPF <- FPF[!dups]
+                     TTT <- TTT[!dups]
+                     
+                     if (!is.null(max.num.points)) {
+                       TPF <- TPF[seq(from = 1, to = length(TPF), length.out = max.num.points)]
+                       FPF <- FPF[seq(from = 1, to = length(FPF), length.out = max.num.points)]
+                       TTT <- TTT[seq(from = 1, to = length(TTT), length.out = max.num.points)]
+                     }
+                     
+                     tp <- c(0, TPF)/sum(D == 1)
+                     fp <- c(0, FPF)/sum(D == 0)
+                     
+                     if(increasing) Lowest <- Inf else Lowest <- -Inf
+                     cutoffs <- c(Lowest, TTT)
                      
                      data.frame(false_positive_fraction = fp, true_positive_fraction = tp, cutoffs = cutoffs)
-              
+                     
                      
                    })
 
 #' Calculate the empirical Receiver Operating Characteristic curve
 #' 
-#' Given a binary outcome d and continous measurement m, computes the empirical
+#' Given a binary outcome d and continuous measurement m, computes the empirical
 #' ROC curve for assessing the classification accuracy of m
 #' 
 #' @inheritParams ggplot2::stat_identity
 #' @param na.rm Remove missing observations
+#' @param max.num.points maximum number of points to plot
+#' @param increasing TRUE (default) if M is positively associated with Pr(D = 1), if FALSE, assumes M is negatively associated with Pr(D = 1)
 #' @section Aesthetics:
 #' \code{stat_roc} understands the following aesthetics (required aesthetics
 #' are in bold):
@@ -59,10 +101,10 @@ StatRoc <- ggproto("StatRoc", Stat,
 #'   \item \strong{\code{m}} The continuous biomarker/predictor
 #'   \item \strong{\code{d}} The binary outcome, if not coded as 0/1, the 
 #'   smallest level in sort order is assumed to be 0, with a warning
-#'   \item \code{alpha}
+#'   \item \code{alpha} Controls the label alpha, see also \code{linealpha} and \code{pointalpha}
 #'   \item \code{color}
-#'   \item \code{linetype}
-#'   \item \code{size}
+#'   \item \code{linetype} 
+#'   \item \code{size} Controls the line weight, see also \code{pointsize} and \code{labelsize}
 #' }
 #' @section Computed variables:
 #' \describe{
@@ -81,7 +123,7 @@ StatRoc <- ggproto("StatRoc", Stat,
 #' ggplot(rocdata, aes(m = M, d = D)) + stat_roc()
 
 stat_roc <- function(mapping = NULL, data = NULL, geom = "roc",
-                         position = "identity", show.legend = NA, inherit.aes = TRUE, na.rm = TRUE, ...) {
+                     position = "identity", show.legend = NA, inherit.aes = TRUE, na.rm = TRUE, max.num.points = 1e3, increasing = TRUE, ...) {
   layer(
     stat = StatRoc,
     data = data,
@@ -90,7 +132,7 @@ stat_roc <- function(mapping = NULL, data = NULL, geom = "roc",
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
-    params = list(na.rm = na.rm, ...)
+    params = list(na.rm = na.rm, max.num.points = max.num.points, increasing = TRUE, ...)
   )
   
 }
@@ -100,13 +142,17 @@ stat_roc <- function(mapping = NULL, data = NULL, geom = "roc",
 #' @param linejoin Line join style (round, mitre, bevel)
 #' @param linemitre Line mitre limit (number greater than 1)
 #' @param arrow Arrow specification, as created by \code{\link[grid]{arrow}}
-#' @param alpha.line Alpha level for the lines
-#' @param alpha.point Alpha level for the cutoff points
-#' @param size.point Size of cutoff points
+#' @param linealpha Alpha level for the lines, alpha.line is deprecated
+#' @param pointalpha Alpha level for the cutoff points, alpha.point is deprecated
+#' @param pointsize Size of cutoff points, size.point is deprecated
 #' @param labels Logical, display cutoff text labels
 #' @param labelsize Size of cutoff text labels
 #' @param labelround Integer, number of significant digits to round cutoff labels
 #' @param na.rm Remove missing values from curve
+#' @param cutoffs.at Vector of user supplied cutoffs to plot as points. If non-NULL, 
+#' it will override the values of n.cuts and plot the observed cutoffs closest to the user-supplied ones.
+#' @param cutoff.labels vector of user-supplied labels for the cutoffs.  Must be a character vector of
+#' the same length as cutoffs.at.
 #' @section Computed variables:
 #' \describe{
 #'   \item{false_positive_fraction}{estimate of false positive fraction}
@@ -126,7 +172,9 @@ stat_roc <- function(mapping = NULL, data = NULL, geom = "roc",
 #' ggplot(rocdata, aes(m = M, d = D, color = Z)) + geom_roc()
 #' ggplot(rocdata, aes(m = M, d = D)) + geom_roc() + facet_wrap(~ Z)
 #' ggplot(rocdata, aes(m = M, d = D)) + geom_roc(n.cuts = 20)
+#' ggplot(rocdata, aes(m = M, d = D)) + geom_roc(cutoffs.at = c(1.5, 1, .5, 0, -.5))
 #' ggplot(rocdata, aes(m = M, d = D)) + geom_roc(labels = FALSE)
+#' ggplot(rocdata, aes(m = M, d = D)) + geom_roc(size = 1.25)
 #' }
 
 GeomRoc <- ggproto("GeomRoc", Geom, 
@@ -137,14 +185,29 @@ GeomRoc <- ggproto("GeomRoc", Geom,
                    non_missing_aes = c("size", "shape"),
                    draw_group = function(data, panel_scales, coord, n.cuts = 10, arrow = NULL,
                                          lineend = "butt", linejoin = "round", linemitre = 1, 
-                                         alpha.line = 1, alpha.point = 1,
-                                         size.point = .5, labels = TRUE, labelsize = 3.88, labelround = 1,
-                                         na.rm = TRUE){
+                                         linealpha = 1, pointalpha = 1, size.point, alpha.point, alpha.line, 
+                                         pointsize = .5, labels = TRUE, labelsize = 3.88, labelround = 1,
+                                         na.rm = TRUE, cutoffs.at = NULL, cutoff.labels = NULL, ...){
                      
-                     if(nrow(data) < n.cuts){ 
-                       dex <- 1:nrow(data)
+                     if(!missing(alpha.line)) linealpha <- alpha.line
+                     if(!missing(alpha.point)) pointalpha <- alpha.point
+                     if(!missing(size.point)) pointsize <- size.point
+                     
+                     
+                     if(!is.null(cutoffs.at)) {
+                       ## find the index of the points closest to the supplied cutoffs
+                       dex <- sapply(cutoffs.at, function(x){ 
+                         in.dx <- abs(data$cutoffs - x)
+                         which.min(in.dx)
+                       })
+                       
+                       
                      } else {
-                       dex <- as.integer(seq(1, nrow(data), length.out = n.cuts))
+                       if(nrow(data) < n.cuts){ 
+                         dex <- 1:nrow(data)
+                       } else {
+                         dex <- as.integer(seq(1, nrow(data), length.out = n.cuts))
+                       }
                      }
                      
                      coords <- coord$transform(data, panel_scales)
@@ -154,14 +217,19 @@ GeomRoc <- ggproto("GeomRoc", Geom,
                        pg <- pointsGrob(
                          coordsp$x, coordsp$y,
                          pch = coordsp$shape,
-                         size = unit(size.point, "char"),
+                         size = unit(pointsize, "char"),
                          gp = gpar(
                            col = coordsp$colour,
                            fill = coordsp$fill,
-                           alpha = alpha.point
+                           alpha = pointalpha
                          )
                        )
-                      } else pg <- nullGrob()
+                       
+                      
+                     } else{ 
+                       pg <- nullGrob()
+                      
+                     }
                      
                      keep <- function(x) {
                        # from first non-missing to last non-missing
@@ -219,8 +287,8 @@ GeomRoc <- ggproto("GeomRoc", Geom,
                          munched$x[!end], munched$y[!end], munched$x[!start], munched$y[!start],
                          default.units = "native", arrow = arrow,
                          gp = gpar(
-                           col = alpha(munched$colour, alpha.line)[!end],
-                           fill = alpha(munched$colour, alpha.line)[!end],
+                           col = alpha(munched$colour, linealpha)[!end],
+                           fill = alpha(munched$colour, linealpha)[!end],
                            lwd = munched$size[!end] * .pt,
                            lty = munched$linetype[!end],
                            lineend = lineend,
@@ -234,8 +302,8 @@ GeomRoc <- ggproto("GeomRoc", Geom,
                          munched$x, munched$y, id = id,
                          default.units = "native", arrow = arrow,
                          gp = gpar(
-                           col = alpha(munched$colour, alpha.line)[start],
-                           fill = alpha(munched$colour, alpha.line)[start],
+                           col = alpha(munched$colour, linealpha)[start],
+                           fill = alpha(munched$colour, linealpha)[start],
                            lwd = munched$size[start] * .pt,
                            lty = munched$linetype[start],
                            lineend = lineend,
@@ -245,8 +313,14 @@ GeomRoc <- ggproto("GeomRoc", Geom,
                        )
                      }
                      
-                     if(labels & n.cuts > 0){
-                       lab <- round(coordsp$label, labelround)
+                     if (labels & (n.cuts > 0 | !is.null(cutoffs.at))) {
+                       
+                       if (is.null(cutoff.labels)) {
+                         lab <- round(coordsp$label, labelround)
+                       } else {
+                         lab <- cutoff.labels
+                       }
+                       
                        
                        if (is.character(coordsp$vjust)) {
                          coordsp$vjust <- compute_just(coordsp$vjust, coordsp$y)
@@ -275,7 +349,8 @@ GeomRoc <- ggproto("GeomRoc", Geom,
                      
                      
                    }, 
-                   draw_key = draw_key_path)
+                   draw_key = roc_key 
+                   )
 
 #' Empirical Receiver Operating Characteristic Curve
 #' 
@@ -308,17 +383,20 @@ GeomRoc <- ggproto("GeomRoc", Geom,
 
 geom_roc <- function(mapping = NULL, data = NULL, stat = "roc", n.cuts = 10, arrow = NULL,
                      lineend = "butt", linejoin = "round", linemitre = 1, 
-                     alpha.line = 1, alpha.point = 1,
-                     size.point = .5, labels = TRUE, labelsize = 3.88, labelround = 1,
-                     na.rm = TRUE, position = "identity", show.legend = NA, inherit.aes = TRUE, ...) {
+                     linealpha = 1, pointalpha = 1, 
+                     pointsize = .5, labels = TRUE, labelsize = 3.88, labelround = 1,
+                     na.rm = TRUE, cutoffs.at = NULL, cutoff.labels = NULL, position = "identity", show.legend = NA, inherit.aes = TRUE, ...) {
+  
+  
   layer(
     geom = GeomRoc, mapping = mapping, data = data, stat = stat, 
     position = position, show.legend = show.legend, inherit.aes = inherit.aes, 
     params = list(na.rm = na.rm, n.cuts = n.cuts, arrow = arrow,
-                     lineend = lineend, linejoin = linejoin, linemitre = linemitre, 
-                     alpha.line = alpha.line, alpha.point = alpha.point,
-                     size.point = size.point, labels = labels, labelsize = labelsize, labelround = labelround,
-                    ...)
+                  lineend = lineend, linejoin = linejoin, linemitre = linemitre, 
+                  linealpha = linealpha, pointalpha = pointalpha,
+                  pointsize = pointsize, labels = labels, labelsize = labelsize, labelround = labelround, 
+                  cutoffs.at = cutoffs.at, cutoff.labels = cutoff.labels, ...)
   )
 }
 
+
